/**
 * This code shows how to use state machines in low power foreground / background
 * designs. Fore more information go to www.sinelabore.com
 * The main function initializes everything and then waits in low power mode for
 * the arrival of events.
 *
 * (C) Peter Mueller, Dec 2019
 * Feel free to reuse this code as basis for your projects
 */
#include <msp430.h>
#include <stdint.h>
#include "lib/fifo.h"
#include "lib/timer.h"
#include "lib/serial.h"

// header from the generated state machine
#include "state_machine/tl_sm_ext.h"
#include "state_machine/tl_sm.h"
#include "state_machine/tl_sm_trace.h"


void Software_Trim();            // Software Trim to get the best DCOFTRIM value
#define MCLK_FREQ_MHZ 8                     // MCLK = 1MHz

#define TIMER_INIT 10000

// library variables for a fifo with FIFO_SIZE elements
#define FIFO_SIZE (16U)
FIFO_T fifo; // fifo object
uint8_t fifoRawMem[FIFO_SIZE]; // memory to store fifo msgs in

// state machine instance variable
TL_SM_INSTANCEDATA_T sm_inst = TL_SM_INSTANCEDATA_INIT;


void main(void)
    {
    WDTCTL = WDTPW | WDTHOLD;   // stop watchdog timer
    _EINT();

    // init to 8MHz
    __bis_SR_register(SCG0);                // Disable FLL
    CSCTL3 = SELREF__REFOCLK;               // Set REFO as FLL reference source
    CSCTL1 = DCOFTRIMEN_1 | DCOFTRIM0 | DCOFTRIM1 | DCORSEL_3; // DCOFTRIM=3, DCO Range = 8MHz
    CSCTL2 = FLLD_0 + 243;                   // DCODIV = 8MHz
    __delay_cycles(3);
    __bic_SR_register(SCG0);                // Enable FLL
    Software_Trim();             // Software Trim to get the best DCOFTRIM value
    CSCTL4 = SELMS__DCOCLKDIV | SELA__REFOCLK; // set default REFO(~32768Hz) as ACLK source, ACLK = 32768Hz
                                               // default DCODIV as MCLK and SMCLK source

    PM5CTL0 &= ~LOCKLPM5; // Disable the GPIO power-on default high-impedance mode

    P1DIR |= 0xff;                         // P1 output
    P2DIR |= 0xff;                         // P2 output
    P1OUT = 0;
    P2OUT = 0;

    // Configure UART pins
    P1SEL0 |= BIT6 | BIT7;                  // set 2-UART pin as second function

    TB0CCTL0 |= CCIE;                             // TBCCR0 interrupt enabled
    TB0CCR0 = TIMER_INIT;
    TB0CTL |= TBSSEL__SMCLK | MC__CONTINUOUS;     // SMCLK, continuous mode

    // initializes the software timer module and fifo
    timerInit();
    fifoInit(&fifo, &fifoRawMem[0], FIFO_SIZE);

    // initialize a basic serial interface for debugging / trace purposes
    serialInit();

    // call once the statemachine to init
    tl_sm(&sm_inst, TL_SM_NO_MSG);


    // endless loop where we check for events in the queue.
    // If there are any take them out and process them
    // Then go in low power mode again.
    while (1)
    {
        bool retVal;
        do
        {
            uint8_t msg;

            retVal = fifoGet(&fifo, &msg);
            if (retVal != QUEUE_EMPTY)
            {
                tl_sm(&sm_inst, (TL_SM_EVENT_T) msg);
            }
        }
        while (retVal != QUEUE_EMPTY);
        __bis_SR_register(LPM0_bits + GIE);       // Enter LPM0 w/ interrupt

    }
}


// hardware timer irq handler that calls the software timer module on a cyclic basis.
// In case timeouts have happened the related evens are stored in the
// event queue and the CPU will be woken up to process them.
void __attribute__ ((interrupt(TIMER0_B0_VECTOR))) Timer_B(void)
{
    static uint16_t led_cnt = 0;
    static uint16_t tick_cnt=0;

    TB0CCR0 += TIMER_INIT;                             // Add Offset to TBCCR0

    led_cnt++;
    if (led_cnt >= 800)
    {
        P1OUT ^= BIT0; // Toggle LED on eval board for debugging
        led_cnt = 0;
    }

    tick_cnt++;
    if(tick_cnt>=10){ //80 will give correct timing
        tick_cnt=0;
        if (tick())
        { // timeout from application found, wakeup main task
            __bic_SR_register_on_exit(LPM0_bits);   // Exit active CPU
        }
    }

}


// Trim clock system
// (C) TI
void Software_Trim()
{
    unsigned int oldDcoTap = 0xffff;
    unsigned int newDcoTap = 0xffff;
    unsigned int newDcoDelta = 0xffff;
    unsigned int bestDcoDelta = 0xffff;
    unsigned int csCtl0Copy = 0;
    unsigned int csCtl1Copy = 0;
    unsigned int csCtl0Read = 0;
    unsigned int csCtl1Read = 0;
    unsigned int dcoFreqTrim = 3;
    unsigned char endLoop = 0;

    do
    {
        CSCTL0 = 0x100;                         // DCO Tap = 256
        do
        {
            CSCTL7 &= ~DCOFFG;                  // Clear DCO fault flag
        }while (CSCTL7 & DCOFFG);               // Test DCO fault flag

        __delay_cycles((unsigned int)3000 * MCLK_FREQ_MHZ);// Wait FLL lock status (FLLUNLOCK) to be stable
                                                           // Suggest to wait 24 cycles of divided FLL reference clock
        while((CSCTL7 & (FLLUNLOCK0 | FLLUNLOCK1)) && ((CSCTL7 & DCOFFG) == 0));

        csCtl0Read = CSCTL0;                   // Read CSCTL0
        csCtl1Read = CSCTL1;                   // Read CSCTL1

        oldDcoTap = newDcoTap;                 // Record DCOTAP value of last time
        newDcoTap = csCtl0Read & 0x01ff;       // Get DCOTAP value of this time
        dcoFreqTrim = (csCtl1Read & 0x0070)>>4;// Get DCOFTRIM value

        if(newDcoTap < 256)                    // DCOTAP < 256
        {
            newDcoDelta = 256 - newDcoTap;     // Delta value between DCPTAP and 256
            if((oldDcoTap != 0xffff) && (oldDcoTap >= 256)) // DCOTAP cross 256
                endLoop = 1;                   // Stop while loop
            else
            {
                dcoFreqTrim--;
                CSCTL1 = (csCtl1Read & (~DCOFTRIM)) | (dcoFreqTrim<<4);
            }
        }
        else                                   // DCOTAP >= 256
        {
            newDcoDelta = newDcoTap - 256;     // Delta value between DCPTAP and 256
            if(oldDcoTap < 256)                // DCOTAP cross 256
                endLoop = 1;                   // Stop while loop
            else
            {
                dcoFreqTrim++;
                CSCTL1 = (csCtl1Read & (~DCOFTRIM)) | (dcoFreqTrim<<4);
            }
        }

        if(newDcoDelta < bestDcoDelta)         // Record DCOTAP closest to 256
        {
            csCtl0Copy = csCtl0Read;
            csCtl1Copy = csCtl1Read;
            bestDcoDelta = newDcoDelta;
        }

    }while(endLoop == 0);                      // Poll until endLoop == 1

    CSCTL0 = csCtl0Copy;                       // Reload locked DCOTAP
    CSCTL1 = csCtl1Copy;                       // Reload locked DCOFTRIM
    while(CSCTL7 & (FLLUNLOCK0 | FLLUNLOCK1)); // Poll until FLL is locked
}
